//#define F_CPU 8000000UL
#include <util/delay.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>
#include "uart.h"
#include "stepper.h"

char err;

//Receive a '+' or a '-'
char getplusmin(void) {
    char c;
    if (err) return 0;
    c=uart_recv();
    if (c=='+') return 1;
    if (c=='-') return -1;
    err=1;
    return 0;
}

//Receive an [u|U] or a [d|D]
char getupdown(void) {
    char c;
    if (err) return 0;
    c=uart_recv();
    if (c=='u' || c=='U') return 0;
    if (c=='d' || c=='D') return 1;
    err=1;
    return 0;
}

//Receive '0'-'9'
char getdigit(void) {
    char c;
    if (err) return 0;
    c=uart_recv()-'0';
    if ((c<0) || (c>10)) {
	err=1;
	return 0;
    }
    return c;
}

int main(void) {
    char dir1,dir2;
    char c,pen;
    int rm1,rm2;
    uart_init();
    steppers_init();
    uart_send('k');
    
    while(1) {
	sei(); //for the stepper routines
	//get string foirmatted like this:
	//snnnnsnnnnu\n
	//where s is a sign, nnnn is a number being the amount of steps (in decimal)
	//and u is 'u' for pen up or 'd' for pen down.
	err=0;
	dir1=getplusmin();
	rm1=getdigit()*1000;
	rm1+=getdigit()*100;
	rm1+=getdigit()*10;
	rm1+=getdigit()*1;

	dir2=getplusmin();
	rm2=getdigit()*1000;
	rm2+=getdigit()*100;
	rm2+=getdigit()*10;
	rm2+=getdigit()*1;
	pen=getupdown();

	//got an error?
	if (err) {
	    uart_send('?');
	} else {
	    //Nope -> move motors
	    step_motors(rm1,rm2,dir1,dir2,pen);
	}
	while (uart_recv()>32);
	uart_send('K');
    }
}

