<?
//This library will continuously output its current coordinates to a file
//called /tmp/pos, so it can start from there inf interrupted.

//Through this program, lengths are in cm and coordinates for use in the
//motor-calculations are with the top left of the whiteboard as (0,0) and
//x and y being positive in the right/down directions.
$settings=array(
    "m1_x"=>-8.5,"m1_y"=>-11, //motor 1 position
    "m2_x"=>62, "m2_y"=>-10, //motor 2 position
    "m1_steplen"=>32.5/15000, //length of one step of motor 1
    "m2_steplen"=>-32.2/15000, //length of one step of motor 2
    "step"=>5,  //maximum length of one command to be sent to the motors, in cm
    //these get overwritten by a value from /tmp/pos, if it exists.
    "posx"=>27.5, //startpos, x
    "posy"=>3, //startpos, y
);



//Gives the lengths the ropes need to have for the pen to be at a certain point
function calclens($x,$y) {
    global $settings;
    $dx1=$x-$settings["m1_x"];
    $dy1=$y-$settings["m1_y"];
    $dx2=$x-$settings["m2_x"];
    $dy2=$y-$settings["m2_y"];
    $r["l1"]=sqrt(($dx1*$dx1)+($dy1*$dy1));
    $r["l2"]=sqrt(($dx2*$dx2)+($dy2*$dy2));
    return $r;
}

//Send a 'move' command to the serial port.
function send_move($c,$s1,$s2,$pen) {
    static $c1,$c2;
//    $emul=true; //enable not to write anything but just pretend
    $str=sprintf("%+05d%+05d%s\n",$s2,$s1,$pen?"D":"U");
    echo $str;
    if ($emul) {
	$c1+=intval($s1); $c2+=intval($s2);
	echo "Cumul: $c1,$c2\n";
    } else {
	while (!fwrite($c,$str));
	$t=fread($c,1);
	if ($t=="?") {
	    sleep(.1);
	    while (!fwrite($c,$str));
	    $t=fread($c,1);
	}
	if ($t!="K") die("Comm error: $t");
    }
}

//Moves the pen to a certain position
function moveto($c,$x,$y,$pen) {
    static $err_fs1,$err_fs2;
    global $settings;

    echo "Moveto: $x,$y, pen=".($pen?"down":"up")."\n";

    $ol=calclens($settings["posx"],$settings["posy"]);
    $nl=calclens($x,$y);

    $fs1=($nl["l1"]-$ol["l1"])/$settings["m1_steplen"]+$err_fs1;
    $fs2=($nl["l2"]-$ol["l2"])/$settings["m2_steplen"]+$err_fs2;

    //calculate the error we make by rounding fs1 and fs2 to ints so we can
    //correct for that later
    $s1=intval($fs1); $s2=intval($fs2);
    $err_fs1=$fs1-$s1;
    $err_fs2=$fs2-$s2;
    
    //if too long, break into pieces using move_precise
    if (abs($s1)>9999 || abs($s2)>9999) {
	move_precise($c,$x,$y,$pen);
	return;
    }
    if ($s1==0 && $s2==0) return;
    send_move($c,$s1,$s2,$pen);
        
    $settings["posx"]=$x;
    $settings["posy"]=$y;
    
    //save position
    $f=fopen("/tmp/pos","w");
    fwrite($f,$x." ".$y."\n");
    fclose($f);
}

//Break a move to pieces and send it like that
function move_precise($c,$x,$y,$pen) {
    global $settings;
    $ox=$settings["posx"];
    $oy=$settings["posy"];
    $dx=$x-$ox; $dy=$y-$oy;
    $l=sqrt(($dx*$dx)+($dy*$dy));
    for ($p=0; $p<$l; $p+=$settings["step"]) {
	$pnt=$p/$l;
	$rx=$x*$pnt+$ox*(1-$pnt);
	$ry=$y*$pnt+$oy*(1-$pnt);
//	echo "Moving to $rx,$ry\n";
	moveto($c,$rx,$ry,$pen);
    }
    moveto($c,$x,$y,$pen);
}


//Draw an array of coordinates to the screen. Auto-scales.
function wb_draw($com,$drw) {
    $minx=999999; $miny=999999;$maxx=-999999; $maxy=-999999;
    foreach($drw as $k) {
	if ($k["x"]<$minx) $minx=$k["x"];
	if ($k["y"]<$miny) $miny=$k["y"];
	if ($k["x"]>$maxx) $maxx=$k["x"];
	if ($k["y"]>$maxy) $maxy=$k["y"];
    }
    echo "x: $minx-$maxx y:$miny-$maxy\n";
    //scaling factors
    $fx=28.5-14;
    $fy=15;
    $tx=28.5+14;
    $ty=15+20;

    //calculate
    $sc_ox=$fx;
    $sc_oy=$fy;
    $sc_dx=($tx-$fx)/($maxx-$minx);
    $sc_dy=($ty-$fy)/($maxy-$miny);

    foreach($drw as $k) {
	$x=(($k["x"]-$minx)*$sc_dx)+$sc_ox;
	$y=(($k["y"]-$miny)*$sc_dy)+$sc_oy;
	if ($k["d"] || (!isset($k["d"]))) $p=1; else $p=0;
	
	if ($x<$fx) die("X underflow");
	if ($y<$fy) die("Y underflow");
	if ($x>$tx) die("X overflow");
	if ($y>$ty) die("Y overflow");
//	echo "GoTo: $x $y\n";
        move_precise($com,$x,$y,$p);
//	moveto($com,$x,$y,$p);
    }

    moveto($com,28.5,3,0);
}

//Open a connection to the whiteboard
function wb_open() {
    global $settings;

    $f=fopen("/tmp/pos","r");
    if ($f!==false) {
	$r=explode(" ",fgets($f,1024));
	fclose($f);
	$settings["posx"]=$r[0];
	$settings["posy"]=$r[1];
    }

    $com=fopen("/dev/ttyUSB0","r+");
    if (!$com) {
	echo 'Error opening com\n';
	exit(0);
    }
    fwrite($com,"+0000+0000U\n");
    while (fread($com,1)!='K');
    echo "Com-port open.\n";
    return($com);
}


?>